@@ -12,7 +12,7 @@ from django.db import models
 from django.db.models.base import ModelBase
 from django.db.models.manager import ManagerDescriptor
 
-from .manager import PolymorphicManager
+from .managers import PolymorphicManager
 from .query import PolymorphicQuerySet
 
 # PolymorphicQuerySet Q objects (and filter()) support these additional key words.
@@ -26,7 +26,7 @@ except ImportError:
 
 
 ###################################################################################
-### PolymorphicModel meta class
+# PolymorphicModel meta class
 
 class PolymorphicModelBase(ModelBase):
     """
@@ -53,11 +53,11 @@ class PolymorphicModelBase(ModelBase):
     """
 
     def __new__(self, model_name, bases, attrs):
-        #print; print '###', model_name, '- bases:', bases
+        # print; print '###', model_name, '- bases:', bases
 
         # Workaround compatibility issue with six.with_metaclass() and custom Django model metaclasses:
         if not attrs and model_name == 'NewBase':
-            if django.VERSION < (1,5):
+            if django.VERSION < (1, 5):
                 # Let Django fully ignore the class which is inserted in between.
                 # Django 1.5 fixed this, see https://code.djangoproject.com/ticket/19688
                 attrs['__module__'] = 'django.utils.six'
@@ -75,7 +75,7 @@ class PolymorphicModelBase(ModelBase):
 
         # add the managers to the new model
         for source_name, mgr_name, manager in inherited_managers:
-            #print '** add inherited manager from model %s, manager %s, %s' % (source_name, mgr_name, manager.__class__.__name__)
+            # print '** add inherited manager from model %s, manager %s, %s' % (source_name, mgr_name, manager.__class__.__name__)
             new_manager = manager._copy_to_model(new_class)
             if mgr_name == '_default_manager':
                 new_class._default_manager = new_manager
@@ -86,7 +86,7 @@ class PolymorphicModelBase(ModelBase):
         # this value is used by the related objects, restoring access to custom queryset methods on related objects.
         user_manager = self.get_first_user_defined_manager(new_class)
         if user_manager:
-            #print '## add default manager', type(def_mgr)
+            # print '## add default manager', type(def_mgr)
             new_class._default_manager = user_manager._copy_to_model(new_class)
             new_class._default_manager._inherited = False   # the default mgr was defined by the user, not inherited
 
@@ -111,7 +111,7 @@ class PolymorphicModelBase(ModelBase):
         use correct mro, only use managers with _inherited==False (they are of no use),
         skip managers that are overwritten by the user with same-named class attributes (in attrs)
         """
-        #print "** ", self.__name__
+        # print "** ", self.__name__
         add_managers = []
         add_managers_keys = set()
         for base in self.__mro__[1:]:
@@ -147,7 +147,7 @@ class PolymorphicModelBase(ModelBase):
                     continue       # manager with that name already added, skip
                 if manager._inherited:
                     continue             # inherited managers (on the bases) have no significance, they are just copies
-                #print '## {0} {1}'.format(self.__name__, key)
+                # print '## {0} {1}'.format(self.__name__, key)
 
                 if isinstance(manager, PolymorphicManager):  # validate any inherited polymorphic managers
                     self.validate_model_manager(manager, self.__name__, key)
@@ -175,7 +175,7 @@ class PolymorphicModelBase(ModelBase):
         # if there are user defined managers, use first one as _default_manager
         if mgr_list:
             _, manager_name, manager = sorted(mgr_list)[0]
-            #sys.stderr.write( '\n# first user defined manager for model "{model}":\n#  "{mgrname}": {mgr}\n#  manager model: {mgrmodel}\n\n'
+            # sys.stderr.write( '\n# first user defined manager for model "{model}":\n#  "{mgrname}": {mgr}\n#  manager model: {mgrmodel}\n\n'
             #    .format( model=self.__name__, mgrname=manager_name, mgr=manager, mgrmodel=manager.model ) )
             return manager
         return None
@@ -191,9 +191,9 @@ class PolymorphicModelBase(ModelBase):
         # app_label here for PolymorphicModel.
         meta = attrs.get('Meta', None)
         do_app_label_workaround = (meta
-                                    and attrs['__module__'] == 'polymorphic'
-                                    and model_name == 'PolymorphicModel'
-                                    and getattr(meta, 'app_label', None) is None)
+                                   and attrs['__module__'] == 'polymorphic'
+                                   and model_name == 'PolymorphicModel'
+                                   and getattr(meta, 'app_label', None) is None)
 
         if do_app_label_workaround:
             meta.app_label = 'poly_dummy_app_label'
@@ -229,21 +229,20 @@ class PolymorphicModelBase(ModelBase):
     # Django's management command 'dumpdata' relies on non-polymorphic
     # behaviour of the _default_manager. Therefore, we catch any access to _default_manager
     # here and return the non-polymorphic default manager instead if we are called from 'dumpdata.py'
+    # Otherwise, the base objects will be upcasted to polymorphic models, and be outputted as such.
     # (non-polymorphic default manager is 'base_objects' for polymorphic models).
     # This way we don't need to patch django.core.management.commands.dumpdata
     # for all supported Django versions.
-    # TODO: investigate Django how this can be avoided
-    _dumpdata_command_running = False
-    if len(sys.argv) > 1:
-        _dumpdata_command_running = (sys.argv[1] == 'dumpdata')
+    if len(sys.argv) > 1 and sys.argv[1] == 'dumpdata':
+        # manage.py dumpdata is running
 
-    def __getattribute__(self, name):
-        if name == '_default_manager':
-            if self._dumpdata_command_running:
+        def __getattribute__(self, name):
+            if name == '_default_manager':
                 frm = inspect.stack()[1]  # frm[1] is caller file name, frm[3] is caller function name
                 if 'django/core/management/commands/dumpdata.py' in frm[1]:
                     return self.base_objects
-                #caller_mod_name = inspect.getmodule(frm[0]).__name__  # does not work with python 2.4
-                #if caller_mod_name == 'django.core.management.commands.dumpdata':
+                # caller_mod_name = inspect.getmodule(frm[0]).__name__  # does not work with python 2.4
+                # if caller_mod_name == 'django.core.management.commands.dumpdata':
 
-        return super(PolymorphicModelBase, self).__getattribute__(name)
+            return super(PolymorphicModelBase, self).__getattribute__(name)
+    # TODO: investigate Django how this can be avoided
