@@ -35,13 +35,21 @@ def transmogrify(cls, obj):
     else:
         # Run constructor, reassign values
         new = cls()
-        for k,v in obj.__dict__.items():
+        for k, v in obj.__dict__.items():
             new.__dict__[k] = v
     return new
 
 
 ###################################################################################
-### PolymorphicQuerySet
+# PolymorphicQuerySet
+
+def _query_annotations(query):
+    try:
+        return query.annotations
+    except AttributeError:
+        # Django < 1.8
+        return query.aggregates
+
 
 class PolymorphicQuerySet(QuerySet):
     """
@@ -64,7 +72,17 @@ class PolymorphicQuerySet(QuerySet):
         new.polymorphic_disabled = self.polymorphic_disabled
         return new
 
-    def non_polymorphic(self, *args, **kwargs):
+    if django.VERSION >= (1, 7):
+        def as_manager(cls):
+            # Make sure the Django 1.7 way of creating managers works.
+            from .managers import PolymorphicManager
+            manager = PolymorphicManager.from_queryset(cls)()
+            manager._built_with_as_manager = True
+            return manager
+        as_manager.queryset_only = True
+        as_manager = classmethod(as_manager)
+
+    def non_polymorphic(self):
         """switch off polymorphic behaviour for this query.
         When the queryset is evaluated, only objects of the type of the
         base class used for this query are returned."""
@@ -127,11 +145,22 @@ class PolymorphicQuerySet(QuerySet):
         qs = self.non_polymorphic()
         return super(PolymorphicQuerySet, qs).aggregate(*args, **kwargs)
 
-    # Since django_polymorphic 'V1.0 beta2', extra() always returns polymorphic results.^
+    if django.VERSION >= (1, 9):
+        # On Django < 1.9, 'qs.values(...)' returned a new special ValuesQuerySet
+        # object, which our polymorphic modifications didn't apply to.
+        # Starting with Django 1.9, the copy returned by 'qs.values(...)' has the
+        # same class as 'qs', so our polymorphic modifications would apply.
+        # We want to leave values queries untouched, so we set 'polymorphic_disabled'.
+        def _values(self, *args, **kwargs):
+            clone = super(PolymorphicQuerySet, self)._values(*args, **kwargs)
+            clone.polymorphic_disabled = True
+            return clone
+
+    # Since django_polymorphic 'V1.0 beta2', extra() always returns polymorphic results.
     # The resulting objects are required to have a unique primary key within the result set
     # (otherwise an error is thrown).
     # The "polymorphic" keyword argument is not supported anymore.
-    #def extra(self, *args, **kwargs):
+    # def extra(self, *args, **kwargs):
 
     def _get_real_instances(self, base_result_objects):
         """
@@ -187,7 +216,7 @@ class PolymorphicQuerySet(QuerySet):
         for base_object in base_result_objects:
             ordered_id_list.append(base_object.pk)
 
-            # check if id of the result object occeres more than once - this can happen e.g. with base_objects.extra(tables=...)
+            # check if id of the result object occurres more than once - this can happen e.g. with base_objects.extra(tables=...)
             if not base_object.pk in base_result_objects_by_id:
                 base_result_objects_by_id[base_object.pk] = base_object
 
@@ -229,8 +258,8 @@ class PolymorphicQuerySet(QuerySet):
                 if real_class != real_concrete_class:
                     real_object = transmogrify(real_class, real_object)
 
-                if self.query.aggregates:
-                    for anno_field_name in six.iterkeys(self.query.aggregates):
+                if _query_annotations(self.query):
+                    for anno_field_name in six.iterkeys(_query_annotations(self.query)):
                         attr = getattr(base_result_objects_by_id[o_pk], anno_field_name)
                         setattr(real_object, anno_field_name, attr)
 
@@ -245,14 +274,14 @@ class PolymorphicQuerySet(QuerySet):
         resultlist = [results[ordered_id] for ordered_id in ordered_id_list if ordered_id in results]
 
         # set polymorphic_annotate_names in all objects (currently just used for debugging/printing)
-        if self.query.aggregates:
-            annotate_names = six.iterkeys(self.query.aggregates)  # get annotate field list
+        if _query_annotations(self.query):
+            annotate_names = list(six.iterkeys(_query_annotations(self.query)))  # get annotate field list
             for real_object in resultlist:
                 real_object.polymorphic_annotate_names = annotate_names
 
         # set polymorphic_extra_select_names in all objects (currently just used for debugging/printing)
         if self.query.extra_select:
-            extra_select_names = six.iterkeys(self.query.extra_select)  # get extra select field list
+            extra_select_names = list(six.iterkeys(self.query.extra_select))  # get extra select field list
             for real_object in resultlist:
                 real_object.polymorphic_extra_select_names = extra_select_names
 
@@ -279,7 +308,7 @@ class PolymorphicQuerySet(QuerySet):
         if self.polymorphic_disabled:
             for o in base_iter:
                 yield o
-            raise StopIteration
+            return
 
         while True:
             base_result_objects = []
@@ -299,19 +328,20 @@ class PolymorphicQuerySet(QuerySet):
                 yield o
 
             if reached_end:
-                raise StopIteration
+                return
 
     def __repr__(self, *args, **kwargs):
         if self.model.polymorphic_query_multiline_output:
             result = [repr(o) for o in self.all()]
-            return  '[ ' + ',\n  '.join(result) + ' ]'
+            return '[ ' + ',\n  '.join(result) + ' ]'
         else:
             return super(PolymorphicQuerySet, self).__repr__(*args, **kwargs)
 
     class _p_list_class(list):
+
         def __repr__(self, *args, **kwargs):
             result = [repr(o) for o in self]
-            return  '[ ' + ',\n  '.join(result) + ' ]'
+            return '[ ' + ',\n  '.join(result) + ' ]'
 
     def get_real_instances(self, base_result_objects=None):
         "same as _get_real_instances, but make sure that __repr__ for ShowField... creates correct output"
