@@ -19,21 +19,25 @@ sys.stderr.write('Using Django version {0} from {1}\n'.format(
 
 if not settings.configured:
     settings.configure(
-        DEBUG = True,
-        TEMPLATE_DEBUG = True,
-        DATABASES = {
+        DEBUG=True,
+        TEMPLATE_DEBUG=True,
+        DATABASES={
             'default': {
                 'ENGINE': 'django.db.backends.sqlite3',
                 'NAME': ':memory:'
             }
         },
-        TEMPLATE_LOADERS = (
+        TEMPLATE_LOADERS=(
             'django.template.loaders.app_directories.Loader',
         ),
-        TEMPLATE_CONTEXT_PROCESSORS = default_settings.TEMPLATE_CONTEXT_PROCESSORS + (
-            'django.core.context_processors.request',
+        TEMPLATE_CONTEXT_PROCESSORS=(
+            # list() is only needed for older versions of django where this is
+            # a tuple:
+            list(default_settings.TEMPLATE_CONTEXT_PROCESSORS) + [
+                'django.core.context_processors.request',
+            ]
         ),
-        TEST_RUNNER = 'django.test.runner.DiscoverRunner' if django.VERSION >= (1,7) else 'django.test.simple.DjangoTestSuiteRunner',
+        TEST_RUNNER = 'django.test.runner.DiscoverRunner' if django.VERSION >= (1, 7) else 'django.test.simple.DjangoTestSuiteRunner',
         INSTALLED_APPS = (
             'django.contrib.auth',
             'django.contrib.contenttypes',
@@ -46,9 +50,15 @@ if not settings.configured:
         SITE_ID = 3,
     )
 
+DEFAULT_TEST_APPS = [
+    'polymorphic',
+]
+
 
 def runtests():
-    argv = sys.argv[:1] + ['test', 'polymorphic', '--traceback'] + sys.argv[1:]
+    other_args = list(filter(lambda arg: arg.startswith('-'), sys.argv[1:]))
+    test_apps = list(filter(lambda arg: not arg.startswith('-'), sys.argv[1:])) or DEFAULT_TEST_APPS
+    argv = sys.argv[:1] + ['test', '--traceback'] + other_args + test_apps
     execute_from_command_line(argv)
 
 if __name__ == '__main__':
